@@ -27,7 +27,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_dag24.c 1799 2013-03-06 00:28:51Z salcock $
+ * $Id$
  *
  */
 
@@ -472,6 +472,7 @@ static libtrace_eventobj_t trace_event_dag(libtrace_t *trace,
                                 event.type = TRACE_EVENT_PACKET;
                         } else {
                                 /* Do not sleep - try to read another packet */
+                                trace->filtered_packets ++;
 				continue;
                         }
                 } else {
@@ -482,7 +483,7 @@ static libtrace_eventobj_t trace_event_dag(libtrace_t *trace,
                 if (trace->snaplen > 0) {
                         trace_set_capture_length(packet, trace->snaplen);
                 }
-
+                trace->accepted_packets ++;
                 return event;
         } while (1);
 
@@ -518,7 +519,7 @@ static void dag_help(void) {
 
 static struct libtrace_format_t dag = {
         "dag",
-        "$Id: format_dag24.c 1799 2013-03-06 00:28:51Z salcock $",
+        "$Id$",
         TRACE_FORMAT_ERF,
 	dag_probe_filename,		/* probe filename */
 	NULL,				/* probe magic */
