@@ -27,7 +27,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_erf.c 1838 2013-06-27 03:25:21Z salcock $
+ * $Id$
  *
  */
 
@@ -636,7 +636,7 @@ static int erf_write_packet(libtrace_out_t *libtrace,
 
 		/* Flags. Can't do this */
 		memset(&erfhdr.flags,1,sizeof(erfhdr.flags));
-		if (trace_get_direction(packet)!=~0U)
+		if (trace_get_direction(packet)!=TRACE_DIR_UNKNOWN)
 			erfhdr.flags.iface = trace_get_direction(packet);
 
 		if (!find_compatible_linktype(libtrace,packet))
@@ -745,7 +745,7 @@ static struct libtrace_eventobj_t erf_event(struct libtrace_t *libtrace, struct
 	/* If we are being told to replay packets as fast as possible, then
 	 * we just need to read and return the next packet in the trace */
 	if (IN_OPTIONS.real_time) {
-		event.size = erf_read_packet(libtrace, packet);
+		event.size = trace_read_packet(libtrace, packet);
 		if (event.size < 1)
 			event.type = TRACE_EVENT_TERMINATE;
 		else
@@ -789,7 +789,7 @@ static void erf_help(void) {
 
 static struct libtrace_format_t erfformat = {
 	"erf",
-	"$Id: format_erf.c 1838 2013-06-27 03:25:21Z salcock $",
+	"$Id$",
 	TRACE_FORMAT_ERF,
 	NULL,				/* probe filename */
 	erf_probe_magic,		/* probe magic */
@@ -832,7 +832,7 @@ static struct libtrace_format_t erfformat = {
 
 static struct libtrace_format_t rawerfformat = {
 	"rawerf",
-	"$Id: format_erf.c 1838 2013-06-27 03:25:21Z salcock $",
+	"$Id$",
 	TRACE_FORMAT_RAWERF,
 	NULL,				/* probe filename */
 	NULL,		/* probe magic */
