@@ -27,7 +27,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_helper.c 1830 2013-05-14 05:17:57Z salcock $
+ * $Id$
  *
  */
 
@@ -97,7 +97,7 @@ struct libtrace_eventobj_t trace_event_device(struct libtrace_t *trace,
 	 * uses mmapped sockets by default, I've switched over to this 
 	 * solution. */
 
-	do {
+        do {
 		tv.tv_sec = 0;
 		tv.tv_usec = 0;
 		rfds_param = rfds;
@@ -110,8 +110,7 @@ struct libtrace_eventobj_t trace_event_device(struct libtrace_t *trace,
 	} while (ret == -1);
 
 	if (FD_ISSET(event.fd, &rfds_param)) {
-		event.size = trace_read_packet(trace,packet);
-		
+                event.size = trace_read_packet(trace,packet);
 		if (event.size < 1) {
 			/* Covers error and EOF events - terminate rather 
 			 * than report a packet as available */
@@ -262,8 +261,20 @@ io_t *trace_open_file(libtrace_t *trace)
 iow_t *trace_open_file_out(libtrace_out_t *trace, int compress_type, int level, int fileflag)
 {
 	iow_t *io = NULL;
-	assert(level<10);
-	assert(level>=0);
+
+        if (level < 0 || level > 9) {
+                trace_set_err_out(trace, TRACE_ERR_UNSUPPORTED_COMPRESS, 
+                                "Compression level %d is invalid, must be between 0 and 9 inclusive", 
+                                level);
+                return NULL;
+        }
+
+        if (compress_type < 0 || 
+                        compress_type >= TRACE_OPTION_COMPRESSTYPE_LAST) {
+                trace_set_err_out(trace, TRACE_ERR_UNSUPPORTED_COMPRESS,
+                                "Invalid compression type %d", compress_type);
+                return NULL;
+        }
 
 	io = wandio_wcreate(trace->uridata, compress_type, level, fileflag);
 
@@ -319,24 +330,3 @@ void trace_set_err_out(libtrace_out_t *trace,int errcode,const char *msg,...)
 	}
 	va_end(va);
 }
-
-/* Byte swapping functions for various inttypes */
-uint64_t byteswap64(uint64_t num)
-{
-	return (byteswap32((num&0xFFFFFFFF00000000ULL)>>32))
-	      |((uint64_t)byteswap32(num&0x00000000FFFFFFFFULL)<<32);
-}
-
-uint32_t byteswap32(uint32_t num)
-{
-	return ((num&0x000000FFU)<<24)
-		| ((num&0x0000FF00U)<<8)
-		| ((num&0x00FF0000U)>>8)
-		| ((num&0xFF000000U)>>24);
-}
-
-uint16_t byteswap16(uint16_t num)
-{
-	return ((num<<8)&0xFF00)|((num>>8)&0x00FF);
-}
-
