@@ -27,7 +27,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_legacy.c 1825 2013-04-19 05:16:13Z salcock $
+ * $Id$
  *
  */
 
@@ -420,7 +420,7 @@ static int legacynzix_get_wire_length(const libtrace_packet_t *packet) {
 static uint64_t legacy_get_erf_timestamp(const libtrace_packet_t *packet)
 {
 	legacy_ether_t *legacy = (legacy_ether_t*)packet->header;
-	return legacy->ts;
+	return bswap_le_to_host64(legacy->ts);
 }  
 
 static uint32_t ts_cmp(uint32_t ts_a, uint32_t ts_b) {
@@ -513,7 +513,7 @@ static void legacynzix_help(void) {
 
 static struct libtrace_format_t legacyatm = {
 	"legacyatm",
-	"$Id: format_legacy.c 1825 2013-04-19 05:16:13Z salcock $",
+	"$Id$",
 	TRACE_FORMAT_LEGACY_ATM,
 	NULL,				/* probe filename */
 	NULL,				/* probe magic */
@@ -556,7 +556,7 @@ static struct libtrace_format_t legacyatm = {
 
 static struct libtrace_format_t legacyeth = {
 	"legacyeth",
-	"$Id: format_legacy.c 1825 2013-04-19 05:16:13Z salcock $",
+	"$Id$",
 	TRACE_FORMAT_LEGACY_ETH,
 	NULL,				/* probe filename */
 	NULL,				/* probe magic */
@@ -599,7 +599,7 @@ static struct libtrace_format_t legacyeth = {
 
 static struct libtrace_format_t legacypos = {
 	"legacypos",
-	"$Id: format_legacy.c 1825 2013-04-19 05:16:13Z salcock $",
+	"$Id$",
 	TRACE_FORMAT_LEGACY_POS,
 	NULL,				/* probe filename */
 	NULL,				/* probe magic */
@@ -642,7 +642,7 @@ static struct libtrace_format_t legacypos = {
 
 static struct libtrace_format_t legacynzix = {
 	"legacynzix",
-	"$Id: format_legacy.c 1825 2013-04-19 05:16:13Z salcock $",
+	"$Id$",
 	TRACE_FORMAT_LEGACY_NZIX,
 	NULL,				/* probe filename */
 	NULL,				/* probe magic */
