@@ -27,7 +27,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_pcapfile.c 1799 2013-03-06 00:28:51Z salcock $
+ * $Id$
  *
  */
 
@@ -69,7 +69,25 @@ typedef struct pcapfile_header_t {
 		uint32_t sigfigs;        /* timestamp accuracy */
 		uint32_t snaplen;        /* aka "wirelen" */
 		uint32_t network;        /* data link type */
-} pcapfile_header_t; 
+} pcapfile_header_t;
+
+#define MAGIC1      0xa1b2c3d4  /* Original */
+#define MAGIC2      0xa1b23c4d  /* Newer nanosecond format */
+#define MAGIC1_REV  0xd4c3b2a1  /* Reversed byteorder detection */
+#define MAGIC2_REV  0x4d3cb2a1
+
+static inline int header_is_backwards_magic(pcapfile_header_t *header) {
+	return (header->magic_number == MAGIC1_REV || header->magic_number == MAGIC2_REV);
+}
+
+static inline int header_is_magic(pcapfile_header_t *header) {
+	return (header->magic_number == MAGIC1 || header->magic_number == MAGIC2 ||
+	        header_is_backwards_magic(header));
+}
+
+static inline int trace_in_nanoseconds(pcapfile_header_t *header) {
+	return (header->magic_number == MAGIC2 || header->magic_number == MAGIC2_REV);
+}
 
 struct pcapfile_format_data_t {
 	struct {
@@ -98,18 +116,20 @@ static int pcapfile_probe_magic(io_t *io)
 	pcapfile_header_t header;
 	int len;
 	len = wandio_peek(io, &header, sizeof(header));
+
 	/* Is this long enough? */
 	if (len < (int)sizeof(header)) {
 		return 0;
 	}
 	/* Pcap magic? */
-	if (header.magic_number == 0xa1b2c3d4 || header.magic_number == 0xd4c3b2a1) {
+	if (header_is_magic(&header)) {
 		return 1;
 	}
 	/* Nope, not pcap */
 	return 0;
 }
 
+
 static int pcapfile_init_input(libtrace_t *libtrace) {
 	libtrace->format_data = malloc(sizeof(struct pcapfile_format_data_t));
 
@@ -145,7 +165,7 @@ static inline uint16_t swaps(libtrace_t *libtrace, uint16_t num)
 		return num;
 	
 	/* We can use the PCAP magic number to determine the byte order */
-	if (DATA(libtrace)->header.magic_number == 0xd4c3b2a1)
+	if (header_is_backwards_magic(&(DATA(libtrace)->header)))
 		return byteswap16(num);
 
 	return num;
@@ -160,7 +180,7 @@ static inline uint32_t swapl(libtrace_t *libtrace, uint32_t num)
 		return num;
 	
 	/* We can use the PCAP magic number to determine the byte order */
-	if (DATA(libtrace)->header.magic_number == 0xd4c3b2a1)
+	if (header_is_backwards_magic(&(DATA(libtrace)->header)))
 		return byteswap32(num);
 
 	return num;
@@ -196,8 +216,7 @@ static int pcapfile_start_input(libtrace_t *libtrace)
 			return -1;
 		}
 		
-		if (swapl(libtrace,DATA(libtrace)->header.magic_number) != 
-					0xa1b2c3d4) {
+		if (!header_is_magic(&(DATA(libtrace)->header))) {
 			trace_set_err(libtrace,TRACE_ERR_INIT_FAILED,
 					"Not a pcap tracefile (magic=%08x)\n",swapl(libtrace,DATA(libtrace)->header.magic_number));
 			return -1; /* Not a pcap file */
@@ -302,7 +321,7 @@ static int pcapfile_prepare_packet(libtrace_t *libtrace,
 	if ((flags & TRACE_PREP_OWN_BUFFER) == TRACE_PREP_OWN_BUFFER) {
 		packet->buf_control = TRACE_CTRL_PACKET;
 	} else
-                packet->buf_control = TRACE_CTRL_EXTERNAL;
+		packet->buf_control = TRACE_CTRL_EXTERNAL;
 	
 	
 	packet->buffer = buffer;
@@ -406,14 +425,14 @@ static int pcapfile_write_packet(libtrace_out_t *out,
 	
 	/* Silently discard RT metadata packets and packets with an
 	 * unknown linktype. */
-	if (linktype == TRACE_TYPE_NONDATA || linktype == ~0U) {
+	if (linktype == TRACE_TYPE_NONDATA || linktype == TRACE_TYPE_UNKNOWN) {
 		return 0;
 	}
 
 	/* If this packet cannot be converted to a pcap linktype then
 	 * pop off the top header until it can be converted
 	 */
-	while (libtrace_to_pcap_linktype(linktype)==~0U) {
+	while (libtrace_to_pcap_linktype(linktype)==TRACE_DLT_ERROR) {
 		if (!demote_packet(packet)) {
 			trace_set_err_out(out, 
 				TRACE_ERR_NO_CONVERSION,
@@ -455,8 +474,9 @@ static int pcapfile_write_packet(libtrace_out_t *out,
 				&pcaphdr, sizeof(pcaphdr));
 	}
 
-	hdr.ts_sec = tv.tv_sec;
-	hdr.ts_usec = tv.tv_usec;
+
+	hdr.ts_sec = (uint32_t)tv.tv_sec;
+	hdr.ts_usec = (uint32_t)tv.tv_usec;
 	hdr.caplen = trace_get_capture_length(packet);
 	assert(hdr.caplen < LIBTRACE_PACKET_BUFSIZE);
 	/* PCAP doesn't include the FCS in its wire length value, but we do */
@@ -593,7 +613,29 @@ static struct timeval pcapfile_get_timeval(
 	
 	hdr = (libtrace_pcapfile_pkt_hdr_t*)packet->header;
 	ts.tv_sec = swapl(packet->trace,hdr->ts_sec);
-	ts.tv_usec = swapl(packet->trace,hdr->ts_usec);
+	/* Check trace is not a dummy calling trace_in_nanoseconds */
+	if (DATA(packet->trace) && trace_in_nanoseconds(&DATA(packet->trace)->header))
+		ts.tv_usec = swapl(packet->trace, hdr->ts_usec) / 1000;
+	else
+		ts.tv_usec = swapl(packet->trace,hdr->ts_usec);
+	return ts;
+}
+
+static struct timespec pcapfile_get_timespec(
+		const libtrace_packet_t *packet) 
+{
+	libtrace_pcapfile_pkt_hdr_t *hdr;
+	struct timespec ts;
+	
+	assert(packet->header);
+	
+	hdr = (libtrace_pcapfile_pkt_hdr_t*)packet->header;
+	ts.tv_sec = swapl(packet->trace,hdr->ts_sec);
+	/* Check trace is not a dummy calling trace_in_nanoseconds */
+	if (DATA(packet->trace) && trace_in_nanoseconds(&DATA(packet->trace)->header))
+		ts.tv_nsec = swapl(packet->trace, hdr->ts_usec);
+	else
+		ts.tv_nsec = swapl(packet->trace, hdr->ts_usec) * 1000;
 	return ts;
 }
 
@@ -609,7 +651,7 @@ static int pcapfile_get_capture_length(const libtrace_packet_t *packet) {
 
 static int pcapfile_get_wire_length(const libtrace_packet_t *packet) {
 	libtrace_pcapfile_pkt_hdr_t *pcapptr;
-	 
+
 	assert(packet->header); 
 
 	pcapptr	= (libtrace_pcapfile_pkt_hdr_t *)packet->header;
@@ -670,7 +712,7 @@ static struct libtrace_eventobj_t pcapfile_event(libtrace_t *libtrace, libtrace_
 	 * we just need to read and return the next packet in the trace */
 
 	if (IN_OPTIONS.real_time) {
-		event.size = pcapfile_read_packet(libtrace, packet);
+		event.size = trace_read_packet(libtrace, packet);
 		if (event.size < 1)
 			event.type = TRACE_EVENT_TERMINATE;
 		else
@@ -693,7 +735,7 @@ static void pcapfile_help(void) {
 
 static struct libtrace_format_t pcapfile = {
 	"pcapfile",
-	"$Id: format_pcapfile.c 1799 2013-03-06 00:28:51Z salcock $",
+	"$Id$",
 	TRACE_FORMAT_PCAPFILE,
 	NULL,				/* probe filename */
 	pcapfile_probe_magic,		/* probe magic */
@@ -715,7 +757,7 @@ static struct libtrace_format_t pcapfile = {
 	NULL,				/* set_direction */
 	NULL,				/* get_erf_timestamp */
 	pcapfile_get_timeval,		/* get_timeval */
-	NULL,				/* get_timespec */
+	pcapfile_get_timespec,		/* get_timespec */
 	NULL,				/* get_seconds */
 	NULL,				/* seek_erf */
 	NULL,				/* seek_timeval */
