@@ -27,7 +27,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: libtrace_int.h 1799 2013-03-06 00:28:51Z salcock $
+ * $Id$
  *
  */
 
@@ -40,7 +40,7 @@
  * @author Perry Lorier
  * @author Shane Alcock
  *
- * @version $Id: libtrace_int.h 1799 2013-03-06 00:28:51Z salcock $
+ * @version $Id$
  *
  * All of the structures and functions defined in this header file are intended
  * for internal use within Libtrace only. They should not be exported as part
@@ -58,6 +58,7 @@ extern "C" {
 #include "common.h"
 #include "libtrace.h"
 #include "wandio.h"
+#include "lt_bswap.h"
 
 #ifdef _MSC_VER
 // warning: deprecated function
@@ -146,6 +147,7 @@ int snprintf(char *str, size_t size, const char *format, ...);
 #include "bpf-jit/bpf-jit.h"
 #endif
 
+
 //#define RP_BUFSIZE 65536U
 
 /** Data about the most recent event from a trace file */
@@ -899,72 +901,6 @@ void *trace_get_payload_from_linux_sll(const void *link,
 DLLEXPORT void *trace_get_payload_from_atm(void *link, uint8_t *type, 
 		uint32_t *remaining);
 
-/** Byteswaps a 64-bit value.
- *
- * @param num		The value to be byteswapped.
- * @return The byteswapped 64-bit number
- *
- */
-uint64_t byteswap64(uint64_t num);
-
-/** Byteswaps a 32-bit value.
- *
- * @param num		The value to be byteswapped.
- * @return The byteswapped 32-bit number
- *
- */
-uint32_t byteswap32(uint32_t num);
-
-/** Byteswaps a 16-bit value.
- *
- * @param num		The value to be byteswapped.
- * @return The byteswapped 16-bit number
- *
- */
-uint16_t byteswap16(uint16_t num);
-
-/** @name Byte ordering
- * Macros that define how to convert a value into a particular byte-order 
- *
- * @{
- */
-#if BYTE_ORDER == BIG_ENDIAN
-#define bswap_host_to_be64(num) ((uint64_t)(num))
-#define bswap_host_to_le64(num) byteswap64(num)
-#define bswap_host_to_be32(num) ((uint32_t)(num))
-#define bswap_host_to_le32(num) byteswap32(num)
-#define bswap_host_to_be16(num) ((uint16_t)(num))
-#define bswap_host_to_le16(num) byteswap16(num)
-
-#define bswap_be_to_host64(num) ((uint64_t)(num))
-#define bswap_le_to_host64(num) byteswap64(num)
-#define bswap_be_to_host32(num) ((uint32_t)(num))
-#define bswap_le_to_host32(num) byteswap32(num)
-#define bswap_be_to_host16(num) ((uint16_t)(num))
-#define bswap_le_to_host16(num) byteswap16(num)
-
-/* We use ntoh*() here, because the compiler may
- * attempt to optimise it
- */
-#elif BYTE_ORDER == LITTLE_ENDIAN
-#define bswap_host_to_be64(num) (byteswap64(num))
-#define bswap_host_to_le64(num) ((uint64_t)(num))
-#define bswap_host_to_be32(num) (htonl(num))
-#define bswap_host_to_le32(num) ((uint32_t)(num))
-#define bswap_host_to_be16(num) (htons(num))
-#define bswap_host_to_le16(num) ((uint16_t)(num))
-
-#define bswap_be_to_host64(num) (byteswap64(num))
-#define bswap_le_to_host64(num) ((uint64_t)(num))
-#define bswap_be_to_host32(num) (ntohl(num))
-#define bswap_le_to_host32(num) ((uint32_t)(num))
-#define bswap_be_to_host16(num) (ntohs(num))
-#define bswap_le_to_host16(num) ((uint16_t)(num))
-
-#else
-#error "Unknown byte order"
-#endif
-/** @} */
 
 #ifdef HAVE_BPF
 /* A type encapsulating a bpf filter
@@ -1020,6 +956,10 @@ void atmhdr_constructor(void);
 /** Constructor for the BPF format module */
 void bpf_constructor(void);
 #endif
+#if HAVE_DPDK
+/** Constructor for Intels DPDK format module */
+void dpdk_constructor(void);
+#endif
 
 /** Extracts the RadioTap flags from a wireless link header
  *
