@@ -27,11 +27,12 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: protocols_transport.c 1799 2013-03-06 00:28:51Z salcock $
+ * $Id$
  *
  */
 
 
+#include "libtrace_int.h"
 #include "libtrace.h"
 #include "protocols.h"
 #include "checksum.h"
@@ -380,8 +381,21 @@ DLLEXPORT uint16_t trace_get_source_port(const libtrace_packet_t *packet)
 {
 	uint32_t remaining;
 	uint8_t proto;
-	const struct ports_t *port = 
-		(const struct ports_t*)trace_get_transport((libtrace_packet_t*)packet,
+	struct ports_t *port;
+        uint16_t fragoff;
+        uint8_t more;
+
+        fragoff = trace_get_fragment_offset(packet, &more);
+
+        /* If we're not the first fragment, we're unlikely to be able
+         * to get any useful port numbers from this packet.
+         */
+        if (fragoff != 0)
+                return 0;
+        
+        
+        port = (struct ports_t*)trace_get_transport(
+                        (libtrace_packet_t*)packet,
 			&proto, &remaining);
 
 	/* Snapped too early */
@@ -389,7 +403,7 @@ DLLEXPORT uint16_t trace_get_source_port(const libtrace_packet_t *packet)
 		return 0;
 
 	/* ICMP *technically* doesn't have ports */
-	if (proto == TRACE_IPPROTO_ICMP)
+	if (proto == TRACE_IPPROTO_ICMP || proto == TRACE_IPPROTO_ICMPV6)
 		return 0;
 
 	if (port)
@@ -403,15 +417,28 @@ DLLEXPORT uint16_t trace_get_destination_port(const libtrace_packet_t *packet)
 {
 	uint32_t remaining;
 	uint8_t proto;
-	struct ports_t *port = 
-		(struct ports_t*)trace_get_transport((libtrace_packet_t*)packet,
+	struct ports_t *port;
+        uint16_t fragoff;
+        uint8_t more;
+
+        fragoff = trace_get_fragment_offset(packet, &more);
+
+        /* If we're not the first fragment, we're unlikely to be able
+         * to get any useful port numbers from this packet.
+         */
+        if (fragoff != 0)
+                return 0;
+        
+        
+        port = (struct ports_t*)trace_get_transport(
+                        (libtrace_packet_t*)packet,
 			&proto, &remaining);
 	/* Snapped too early */
 	if (remaining<4)
 		return 0;
 	
 	/* ICMP *technically* doesn't have ports */
-	if (proto == TRACE_IPPROTO_ICMP)
+	if (proto == TRACE_IPPROTO_ICMP || proto == TRACE_IPPROTO_ICMPV6)
 		return 0;
 
 	if (port)
