@@ -27,7 +27,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: trace.c 1799 2013-03-06 00:28:51Z salcock $
+ * $Id$
  *
  */
 
@@ -47,6 +47,7 @@
 #include <sys/socket.h>
 #endif
 #include <stdarg.h>
+#include <sys/param.h>
 
 #ifdef HAVE_LIMITS_H
 #  include <limits.h>
@@ -144,6 +145,9 @@ static void trace_init(void)
 		rt_constructor();
 #ifdef HAVE_DAG
 		dag_constructor();
+#endif
+#ifdef HAVE_DPDK
+        dpdk_constructor();
 #endif
 	}
 }
@@ -718,6 +722,7 @@ DLLEXPORT int trace_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet)
 	if (libtrace->format->read_packet) {
 		do {
 			size_t ret;
+                        int filtret;
 			/* Clear the packet cache */
 			trace_clear_cache(packet);
 			ret=libtrace->format->read_packet(libtrace,packet);
@@ -728,7 +733,13 @@ DLLEXPORT int trace_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet)
 				/* If the filter doesn't match, read another
 				 * packet
 				 */
-				if (!trace_apply_filter(libtrace->filter,packet)){
+                                filtret = trace_apply_filter(libtrace->filter, packet);
+                                if (filtret == -1) {
+                                        /* Error compiling filter, probably */
+                                        return ~0U;
+                                }
+                                
+                                if (filtret == 0) {
 					++libtrace->filtered_packets;
 					continue;
 				}
@@ -920,13 +931,7 @@ DLLEXPORT struct timeval trace_get_timeval(const libtrace_packet_t *packet) {
 	} else if (packet->trace->format->get_erf_timestamp) {
 		/* timestamp -> timeval */
 		ts = packet->trace->format->get_erf_timestamp(packet);
-#if __BYTE_ORDER == __BIG_ENDIAN
-		tv.tv_sec = ts & 0xFFFFFFFF;
-#elif __BYTE_ORDER == __LITTLE_ENDIAN
 		tv.tv_sec = ts >> 32;
-#else
-#error "What on earth are you running this on?"
-#endif
 		tv.tv_usec = ((ts&0xFFFFFFFF)*1000000)>>32;
        		if (tv.tv_usec >= 1000000) {
                		tv.tv_usec -= 1000000;
@@ -958,13 +963,7 @@ DLLEXPORT struct timespec trace_get_timespec(const libtrace_packet_t *packet) {
 	} else if (packet->trace->format->get_erf_timestamp) {
 		/* timestamp -> timeval */
 		uint64_t erfts = packet->trace->format->get_erf_timestamp(packet);
-#if __BYTE_ORDER == __BIG_ENDIAN
-		ts.tv_sec = erfts & 0xFFFFFFFF;
-#elif __BYTE_ORDER == __LITTLE_ENDIAN
 		ts.tv_sec = erfts >> 32;
-#else
-#error "What on earth are you running this on?"
-#endif
 		ts.tv_nsec = ((erfts&0xFFFFFFFF)*1000000000)>>32;
        		if (ts.tv_nsec >= 1000000000) {
                		ts.tv_nsec -= 1000000000;
@@ -1082,7 +1081,7 @@ DLLEXPORT libtrace_linktype_t trace_get_link_type(const libtrace_packet_t *packe
 
 	if (packet->link_type == 0) {
 		if (!packet->trace->format->get_link_type)
-			return (libtrace_linktype_t)-1;
+			return TRACE_TYPE_UNKNOWN;
 		((libtrace_packet_t *)packet)->link_type =
 			packet->trace->format->get_link_type(packet);
 	}
@@ -1119,10 +1118,11 @@ DLLEXPORT libtrace_eventobj_t trace_event(libtrace_t *trace,
 	packet->trace = trace;
 
 	if (packet->trace->format->trace_event) {
-		event=packet->trace->format->trace_event(trace,packet);
-		if (event.type == TRACE_EVENT_PACKET) {
-			++trace->accepted_packets;
-		}
+		/* Note: incrementing accepted, filtered etc. packet
+                 * counters is handled by the format-specific 
+                 * function so don't increment them here.
+                 */
+                event=packet->trace->format->trace_event(trace,packet);
 	}
 	return event;
 
@@ -1223,7 +1223,7 @@ static int trace_bpf_compile(libtrace_filter_t *filter,
 					"Packet has an unknown linktype");
 			return -1;
 		}
-		if (libtrace_to_pcap_dlt(linktype) == ~1U) {
+		if (libtrace_to_pcap_dlt(linktype) == TRACE_DLT_ERROR) {
 			trace_set_err(packet->trace,TRACE_ERR_BAD_FILTER,
 					"Unknown pcap equivalent linktype");
 			return -1;
@@ -1274,7 +1274,7 @@ DLLEXPORT int trace_apply_filter(libtrace_filter_t *filter,
 	if (linktype == TRACE_TYPE_NONDATA)
 		return 1;	
 
-	if (libtrace_to_pcap_dlt(linktype)==~0U) {
+	if (libtrace_to_pcap_dlt(linktype)==TRACE_DLT_ERROR) {
 		
 		/* If we cannot get a suitable DLT for the packet, it may
 		 * be because the packet is encapsulated in a link type that
@@ -1287,7 +1287,7 @@ DLLEXPORT int trace_apply_filter(libtrace_filter_t *filter,
 		packet_copy=trace_copy_packet(packet);
 		free_packet_needed=true;
 
-		while (libtrace_to_pcap_dlt(linktype) == ~0U) {
+		while (libtrace_to_pcap_dlt(linktype) == TRACE_DLT_ERROR) {
 			if (!demote_packet(packet_copy)) {
 				trace_set_err(packet->trace, 
 						TRACE_ERR_NO_CONVERSION,
@@ -1635,12 +1635,13 @@ DLLEXPORT int trace_seek_erf_timestamp(libtrace_t *trace, uint64_t ts)
 			struct timeval tv;
 #if __BYTE_ORDER == __BIG_ENDIAN
 			tv.tv_sec = ts & 0xFFFFFFFF;
+			tv.tv_usec = ((ts >> 32) * 1000000) & 0xFFFFFFFF;
 #elif __BYTE_ORDER == __LITTLE_ENDIAN
 			tv.tv_sec = ts >> 32;
+			tv.tv_usec = ((ts&0xFFFFFFFF)*1000000)>>32;
 #else
 #error "What on earth are you running this on?"
 #endif
-			tv.tv_usec = ((ts&0xFFFFFFFF)*1000000)>>32;
 			if (tv.tv_usec >= 1000000) {
 				tv.tv_usec -= 1000000;
 				tv.tv_sec += 1;
@@ -1816,14 +1817,6 @@ uint64_t trace_get_filtered_packets(libtrace_t *trace)
 		return trace->format->get_filtered_packets(trace)+
 			trace->filtered_packets;
 	}
-	if (trace->format->get_received_packets
-		&& trace->format->get_dropped_packets) {
-		return 
-			((trace_get_received_packets(trace)
-			-trace_get_accepted_packets(trace))
-			-trace_get_dropped_packets(trace))
-			+trace->filtered_packets;
-	}
 	return trace->filtered_packets;
 }
 
