@@ -33,7 +33,7 @@ char *output_base = NULL;
 static char *strdupcat(char *str,char *app)
 {
 	str=realloc(str,strlen(str)+strlen(app)+1);
-	strcat(str,app);
+	strncat(str,app,strlen(str) + strlen(app));
 	return str;
 }
 
@@ -315,6 +315,8 @@ int main(int argc, char *argv[])
 		compress_type = TRACE_OPTION_COMPRESSTYPE_BZ2;
 	} else if (strncmp(compress_type_str, "lzo", 3) == 0) {
 		compress_type = TRACE_OPTION_COMPRESSTYPE_LZO;
+	} else if (strncmp(compress_type_str, "xz", 2) == 0) {
+		compress_type = TRACE_OPTION_COMPRESSTYPE_LZMA;
 	} else if (strncmp(compress_type_str, "no", 2) == 0) {
 		compress_type = TRACE_OPTION_COMPRESSTYPE_NONE;
 	} else {
